@@ -61,6 +61,8 @@ using namespace exceptions;
 
 int verbose = 0;
 const char * menuencoding = "UTF-8";
+bool is_root;
+const char *pw_dir;
 
 map<string, functions::func *> func_data;
 
@@ -477,7 +479,7 @@ bool supportedinfo::supports(string& name)
 
 void read_forcetree(parsestream &i)
 {
-  Regex r("[a-zA-Z/-_ ]");
+  Regex r("[] /0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\^_abcdefghijklmnopqrstuvwxyz]");
 
   while(1)
   {
@@ -640,16 +642,8 @@ methodinfo::methodinfo(parsestream &i)
   if (outputenc == "LOCALE")
       outputenc = nl_langinfo(CODESET);
   
-  check_vars();
-}
-
-void methodinfo::check_vars()
-{
-  if(!(genmenu && startmenu && endmenu)) {
-    /* don't translate genmenu, startmenu, endmenu and don't change \"\" */
-    cerr << _("install-menu: At least one of genmenu, startmenu, endmenu\n"
-        "is undefined. All of these have to be defined \n"
-        "(although they may be equal to \"\").\n");
+  if (!genmenu) {
+    cerr << String::compose(_("install-menu: %1 must be defined in menu-method %2"), "genmenu", i.filename());
     throw informed_fatal();
   }
 }
@@ -670,9 +664,15 @@ cat_str *methodinfo::rootprefix()
 
 string methodinfo::prefix()
 {
-  if (getuid()) {
-    struct passwd *pw = getpwuid(getuid());
-    return string(pw->pw_dir)+"/"+userprefix()->soutput(root_menu.vars);
+  if (!is_root) {
+    string up = userprefix()->soutput(root_menu.vars);
+    // When userprefix is prefixed by // instead of just one /, treat the
+    // URL as absolute instead of relative.
+    if (up.length() > 1 && up.substr(0, 2) == "//") {
+      return up.substr(1);
+    } else {
+      return string(pw_dir)+"/"+up;
+    }
   } else {
     return rootprefix()->soutput(root_menu.vars);
   }
@@ -803,7 +803,7 @@ void includemenus(string replace, string menu_filename)
 
   ifstream input_file(input_filename.c_str());
   if (!input_file) {
-    if (getuid()) {
+    if (!is_root) {
       // Running as non-root:
       string input_filename2 = menumethod->rootprefix()->soutput(root_menu.vars) + '/' + menumethod->examplercfile();
 
@@ -834,7 +834,7 @@ void includemenus(string replace, string menu_filename)
 
   if (!output_file) {
     cerr << String::compose(_("Cannot open file %1.\n"), output_filename); 
-    if (getuid())
+    if (!is_root)
         cerr << _("In order to be able to create the user config file(s) for the window manager,\n" 
             "the above file needs to be writeable (and/or the directory needs to exist).\n");
     throw informed_fatal();
@@ -892,6 +892,10 @@ void usage(ostream &c)
 
 int main(int argc, char **argv)
 {
+  uid_t uid = getuid();
+  struct passwd *pw = getpwuid(uid);
+  is_root = (uid == 0);
+  pw_dir = pw->pw_dir;
   std::string script_name;
   parsestream *ps = 0, *psscript = 0;
   
@@ -900,7 +904,7 @@ int main(int argc, char **argv)
   bind_textdomain_codeset("menu-sections", menuencoding);
   textdomain(PACKAGE);
 
-  if (!getuid()) {
+  if (is_root) {
     // When we are root, we set umask to 022 to ignore the real root umask.
     // This is to ensure that the menu file can be read by users.
     umask(0022);
@@ -929,7 +933,7 @@ int main(int argc, char **argv)
     add_functions();
     
     string directory;
-    if (getuid())
+    if (!is_root)
         directory = MENUMETHODS;
 
     ps = new parsestream(script_name, directory);
@@ -939,9 +943,9 @@ int main(int argc, char **argv)
       throw informed_fatal();
     }
     menumethod = new methodinfo(*ps);
-    if ((menumethod->onlyrunasroot || menumethod->userpref == 0) && getuid())
+    if ((menumethod->onlyrunasroot || menumethod->userpref == 0) && !is_root)
         return 0;
-    if ((menumethod->onlyrunasuser || menumethod->rootpref == 0) && !getuid())
+    if ((menumethod->onlyrunasuser || menumethod->rootpref == 0) && is_root)
         return 0;
     if (menumethod->prerun)
         system((menumethod->prerun->soutput(root_menu.vars)).c_str());
